@@ -1,4 +1,4 @@
-# Copyright (C) 2009--2014  Kipp Cannon
+# Copyright (C) 2009--2018  Kipp Cannon
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the
@@ -24,6 +24,13 @@
 #
 
 
+from __future__ import print_function
+
+
+try:
+	from fpconst import NegInf
+except ImportError:
+	NegInf = float("-inf")
 import itertools
 import math
 import scipy.stats
@@ -31,22 +38,23 @@ import sys
 
 
 import lal
+from lal import rate
 
 
 from glue import segmentsUtils
+from glue.ligolw import ligolw
+from glue.ligolw import array as ligolw_array
+from glue.ligolw import param as ligolw_param
 from glue.ligolw import lsctables
-from glue.ligolw import utils
+from glue.ligolw import utils as ligolw_utils
 from glue.ligolw.utils import process as ligolw_process
 from glue.offsetvector import offsetvector
-from . import ligolw_burca_tailor
-from . import git_version
-from pylal import rate
-from pylal import snglcoinc
+from . import snglcoinc
 
 
 __author__ = "Kipp Cannon <kipp.cannon@ligo.org>"
-__version__ = "git id %s" % git_version.id
-__date__ = git_version.date
+from git_version import date as __date__
+from git_version import version as __version__
 
 
 #
@@ -94,85 +102,167 @@ def triangulators(timing_uncertainties):
 	return triangulators
 
 
+#
+# A binning for instrument combinations
+#
+# FIXME:  we decided that the coherent and null stream naming convention
+# would look like
+#
+# H1H2:LSC-STRAIN_HPLUS, H1H2:LSC-STRAIN_HNULL
+#
+# and so on.  i.e., the +, x and null streams from a coherent network would
+# be different channels from a single instrument whose name would be the
+# mash-up of the names of the instruments in the network.  that is
+# inconsisntent with the "H1H2+", "H1H2-" shown here, so this needs to be
+# fixed but I don't know how.  maybe it'll go away before it needs to be
+# fixed.
+#
+
+
+def InstrumentBins(names = ("E0", "E1", "E2", "E3", "G1", "H1", "H2", "H1H2+", "H1H2-", "L1", "V1")):
+	"""
+	Example:
+
+	>>> x = InstrumentBins()
+	>>> x[frozenset(("H1", "L1"))]
+	55
+	>>> x.centres()[55]
+	frozenset(['H1', 'L1'])
+	"""
+	return rate.HashableBins(frozenset(combo) for n in range(len(names) + 1) for combo in itertools.combinations(names, n))
+
+
 #
 # Parameter distributions
 #
 
 
-def dt_binning(instrument1, instrument2):
-	dt = 0.005 + snglcoinc.light_travel_time(instrument1, instrument2)	# seconds
-	return rate.NDBins((rate.ATanBins(-dt, +dt, 801),))
-
-
-class StringCoincParamsDistributions(snglcoinc.CoincParamsDistributions):
-	ligo_lw_name_suffix = u"stringcusp_coincparamsdistributions"
-
-	binnings = {
-		"H1_snr2_chi2": rate.NDBins((rate.ATanLogarithmicBins(10, 1e7, 801), rate.ATanLogarithmicBins(.1, 1e4, 801))),
-		"H2_snr2_chi2": rate.NDBins((rate.ATanLogarithmicBins(10, 1e7, 801), rate.ATanLogarithmicBins(.1, 1e4, 801))),
-		"L1_snr2_chi2": rate.NDBins((rate.ATanLogarithmicBins(10, 1e7, 801), rate.ATanLogarithmicBins(.1, 1e4, 801))),
-		"V1_snr2_chi2": rate.NDBins((rate.ATanLogarithmicBins(10, 1e7, 801), rate.ATanLogarithmicBins(.1, 1e4, 801))),
-		"H1_H2_dt": dt_binning("H1", "H2"),
-		"H1_L1_dt": dt_binning("H1", "L1"),
-		"H1_V1_dt": dt_binning("H1", "V1"),
-		"H2_L1_dt": dt_binning("H2", "L1"),
-		"H2_V1_dt": dt_binning("H2", "V1"),
-		"L1_V1_dt": dt_binning("L1", "V1"),
-		"H1_H2_dA": rate.NDBins((rate.ATanBins(-0.5, +0.5, 801),)),
-		"H1_L1_dA": rate.NDBins((rate.ATanBins(-0.5, +0.5, 801),)),
-		"H1_V1_dA": rate.NDBins((rate.ATanBins(-0.5, +0.5, 801),)),
-		"H2_L1_dA": rate.NDBins((rate.ATanBins(-0.5, +0.5, 801),)),
-		"H2_V1_dA": rate.NDBins((rate.ATanBins(-0.5, +0.5, 801),)),
-		"L1_V1_dA": rate.NDBins((rate.ATanBins(-0.5, +0.5, 801),)),
-		"H1_H2_df": rate.NDBins((rate.ATanBins(-0.2, +0.2, 501),)),
-		"H1_L1_df": rate.NDBins((rate.ATanBins(-0.2, +0.2, 501),)),
-		"H1_V1_df": rate.NDBins((rate.ATanBins(-0.2, +0.2, 501),)),
-		"H2_L1_df": rate.NDBins((rate.ATanBins(-0.2, +0.2, 501),)),
-		"H2_V1_df": rate.NDBins((rate.ATanBins(-0.2, +0.2, 501),)),
-		"L1_V1_df": rate.NDBins((rate.ATanBins(-0.2, +0.2, 501),)),
+class LnLRDensity(snglcoinc.LnLRDensity):
+	# FIXME:  the interps dictionary maintained here should be
+	# eliminated in favour of an internal mechanism within the PDFs
+	# themselves that performs the interpolation on-the-fly, without
+	# requiring an intermediate object to be created
+	def __init__(self, instruments):
+		self.densities = {}
+		for instrument in instruments:
+			self.densities["%s_snr2_chi2" % instrument] = rate.BinnedLnPDF(rate.NDBins((rate.ATanLogarithmicBins(10, 1e7, 801), rate.ATanLogarithmicBins(.1, 1e4, 801))))
+		for pair in itertools.combinations(sorted(instruments), 2):
+			dt = 0.005 + snglcoinc.light_travel_time(*pair)	# seconds
+			self.densities["%s_%s_dt" % pair] = rate.BinnedLnPDF(rate.NDBins((rate.ATanBins(-dt, +dt, 801),)))
+			self.densities["%s_%s_dA" % pair] = rate.BinnedLnPDF(rate.NDBins((rate.ATanBins(-0.5, +0.5, 801),)))
+			self.densities["%s_%s_df" % pair] = rate.BinnedLnPDF(rate.NDBins((rate.ATanBins(-0.2, +0.2, 501),)))
 		# only non-negative rss timing residual bins will be used
 		# but we want a binning that's linear at the origin so
 		# instead of inventing a new one we just use atan bins that
 		# are symmetric about 0
-		"instrumentgroup,rss_timing_residual": rate.NDBins((snglcoinc.InstrumentBins(names = ("H1", "H2", "L1", "V1")), rate.ATanBins(-0.02, +0.02, 1001)))
-	}
+		self.densities["instrumentgroup,rss_timing_residual"] = rate.BinnedLnPDF(rate.NDBins((InstrumentBins(names = instruments), rate.ATanBins(-0.02, +0.02, 1001))))
 
-	filters = {
-		"H1_snr2_chi2": rate.gaussian_window(11, 11, sigma = 20),
-		"H2_snr2_chi2": rate.gaussian_window(11, 11, sigma = 20),
-		"L1_snr2_chi2": rate.gaussian_window(11, 11, sigma = 20),
-		"V1_snr2_chi2": rate.gaussian_window(11, 11, sigma = 20),
-		"H1_H2_dt": rate.gaussian_window(11, sigma = 20),
-		"H1_L1_dt": rate.gaussian_window(11, sigma = 20),
-		"H1_V1_dt": rate.gaussian_window(11, sigma = 20),
-		"H2_L1_dt": rate.gaussian_window(11, sigma = 20),
-		"H2_V1_dt": rate.gaussian_window(11, sigma = 20),
-		"L1_V1_dt": rate.gaussian_window(11, sigma = 20),
-		"H1_H2_dA": rate.gaussian_window(11, sigma = 20),
-		"H1_L1_dA": rate.gaussian_window(11, sigma = 20),
-		"H1_V1_dA": rate.gaussian_window(11, sigma = 20),
-		"H2_L1_dA": rate.gaussian_window(11, sigma = 20),
-		"H2_V1_dA": rate.gaussian_window(11, sigma = 20),
-		"L1_V1_dA": rate.gaussian_window(11, sigma = 20),
-		"H1_H2_df": rate.gaussian_window(11, sigma = 20),
-		"H1_L1_df": rate.gaussian_window(11, sigma = 20),
-		"H1_V1_df": rate.gaussian_window(11, sigma = 20),
-		"H2_L1_df": rate.gaussian_window(11, sigma = 20),
-		"H2_V1_df": rate.gaussian_window(11, sigma = 20),
-		"L1_V1_df": rate.gaussian_window(11, sigma = 20),
-		# instrument group filter is a no-op, should produce a
-		# 1-bin top-hat window.
-		"instrumentgroup,rss_timing_residual": rate.gaussian_window(1e-100, 11, sigma = 20)
-	}
+	def __call__(self, **params):
+		try:
+			interps = self.interps
+		except AttributeError:
+			self.mkinterps()
+			interps = self.interps
+		return sum(interps[param](*value) for param, value in params.items()) if params else NegInf
+
+	def __iadd__(self, other):
+		if type(self) != type(other) or set(self.densities) != set(other.densities):
+			raise TypeError("cannot add %s and %s" % (type(self), type(other)))
+		for key, pdf in self.densities.items():
+			pdf += other.densities[key]
+		try:
+			del self.interps
+		except AttributeError:
+			pass
+		return self
+
+	def increment(self, weight = 1.0, **params):
+		for param, value in params.items():
+			self.densities[param].count[value] += weight
+
+	def copy(self):
+		new = type(self)([])
+		for key, pdf in self.densities.items():
+			new.densities[key] = pdf.copy()
+		return new
+
+	def mkinterps(self):
+		self.interps = dict((key, pdf.mkinterp()) for key, pdf in self.densities.items() if "rss_timing_residual" not in key)
+
+	def finish(self):
+		for key, pdf in self.densities.items():
+			if key.endswith("_snr2_chi2"):
+				rate.filter_array(pdf.array, rate.gaussian_window(11, 11, sigma = 20))
+			elif key.endswith("_dt") or key.endswith("_dA") or key.endswith("_df"):
+				rate.filter_array(pdf.array, rate.gaussian_window(11, sigma = 20))
+			elif key.startswith("instrumentgroup"):
+				# instrument group filter is a no-op
+				pass
+			else:
+				# shouldn't get here
+				raise Exception
+			pdf.normalize()
+		self.mkinterps()
+
+	def to_xml(self, name):
+		xml = super(LnLRDensity, self).to_xml(name)
+		instruments = set(key.split("_", 1)[0] for key in self.densities if key.endswith("_snr2_chi2"))
+		xml.appendChild(ligolw_param.Param.from_pyvalue("instruments", lsctables.instrumentsproperty.set(instruments)))
+		for key, pdf in self.densities.items():
+			xml.appendChild(pdf.to_xml(key))
+		return xml
+
+	@classmethod
+	def from_xml(cls, xml, name):
+		xml = cls.get_xml_root(xml, name)
+		self = cls(lsctables.instrumentsproperty.get(ligolw_param.get_pyvalue(xml, "instruments")))
+		for key in self.densities:
+			self.densities[key] = rate.BinnedLnPDF.from_xml(xml, key)
+		return self
+
+
+class StringCoincParamsDistributions(snglcoinc.LnLikelihoodRatioMixin):
+	ligo_lw_name_suffix = u"stringcusp_coincparamsdistributions"
+
+	@ligolw_array.use_in
+	@ligolw_param.use_in
+	@lsctables.use_in
+	class LIGOLWContentHandler(ligolw.LIGOLWContentHandler):
+		pass
+
+	def __init__(self, instruments):
+		self.triangulators = triangulators(dict.fromkeys(instruments, 8e-5))
+		self.numerator = LnLRDensity(instruments)
+		self.denominator = LnLRDensity(instruments)
+		self.candidates = LnLRDensity(instruments)
+
+	def __iadd__(self, other):
+		if type(self) != type(other):
+			raise TypeError(other)
+		if set(self.triangulators.keys()) != set(other.triangulators.keys()):
+			raise ValueError("incompatible instruments")
+		self.numerator += other.numerator
+		self.denominator += other.denominator
+		self.candidates += other.candidates
+		return self
+
+	def copy(self):
+		new = type(self)([])
+		new.triangulators = self.triangulators	# share reference
+		new.numerator = self.numerator.copy()
+		new.denominator = self.denominator.copy()
+		new.candidates = self.candidates.copy()
+		return new
+
+	def coinc_params(self, events, offsetvector):
+		params = {}
 
-	@staticmethod
-	def coinc_params(events, offsetvector, triangulators):
 		#
 		# check for coincs that have been vetoed entirely
 		#
 
 		if len(events) < 2:
-			return None
+			return params
 
 		#
 		# Initialize the parameter dictionary, sort the events by
@@ -182,7 +272,6 @@ class StringCoincParamsDistributions(snglcoinc.CoincParamsDistributions):
 		# names
 		#
 
-		params = {}
 		events = tuple(sorted(events, key = lambda event: event.ifo))
 		instruments = tuple(event.ifo for event in events)
 
@@ -190,26 +279,26 @@ class StringCoincParamsDistributions(snglcoinc.CoincParamsDistributions):
 		# zero-instrument parameters
 		#
 
-		ignored, ignored, ignored, rss_timing_residual = triangulators[instruments](tuple(event.peak + offsetvector[event.ifo] for event in events))
-		# FIXME:  rss_timing_residual is forced to 0 to disable this
-		# feature.  all the code to compute it properly is still here and
-		# given suitable initializations, the distribution data is still
-		# two-dimensional and has a suitable filter applied to it, but all
-		# events are forced into the RSS_{\Delta t} = 0 bin, in effect
-		# removing that dimension from the data.  We can look at this again
-		# sometime in the future if we're curious why it didn't help.  Just
-		# delete the next line and you're back in business.
+		ignored, ignored, ignored, rss_timing_residual = self.triangulators[instruments](tuple(event.peak + offsetvector[event.ifo] for event in events))
+		# FIXME:  rss_timing_residual is disabled.  all the code to
+		# compute it properly is still here and given suitable
+		# initializations, the distribution data is still
+		# two-dimensional and has a suitable filter applied to it,
+		# but all events are forced into the RSS_{\Delta t} = 0
+		# bin, in effect removing that dimension from the data.  We
+		# can look at this again sometime in the future if we're
+		# curious why it didn't help.  Just delete the next line
+		# and you're back in business.
 		rss_timing_residual = 0.0
-		params["instrumentgroup,rss_timing_residual"] = (frozenset(instruments), rss_timing_residual)
+		# FIXME:  why is this commented out?
+		#params["instrumentgroup,rss_timing_residual"] = (frozenset(instruments), rss_timing_residual)
 
 		#
 		# one-instrument parameters
 		#
 
 		for event in events:
-			prefix = "%s_" % event.ifo
-
-			params["%ssnr2_chi2" % prefix] = (event.snr**2.0, event.chisq / event.chisq_dof)
+			params["%s_snr2_chi2" % event.ifo] = (event.snr**2.0, event.chisq / event.chisq_dof)
 
 		#
 		# two-instrument parameters.  note that events are sorted by
@@ -239,72 +328,42 @@ class StringCoincParamsDistributions(snglcoinc.CoincParamsDistributions):
 
 		return params
 
-	def add_slidelessbackground(self, database, experiments, param_func_args = ()):
-		# FIXME:  this needs to be taught how to not slide H1 and
-		# H2 with respect to each other
+	def ln_lr_from_triggers(self, events, offsetvector):
+		return self(**self.coinc_params(events, offsetvector))
 
-		# segment lists
-		seglists = database.seglists - database.vetoseglists
+	def finish(self):
+		self.numerator.finish()
+		self.denominator.finish()
+		self.candidates.finish()
+		return self
 
-		# construct the event list dictionary.  remove vetoed
-		# events from the lists and save event peak times so they
-		# can be restored later
-		eventlists = {}
-		orig_peak_times = {}
-		for event in database.sngl_burst_table:
-			if event.peak in seglists[event.ifo]:
-				try:
-					eventlists[event.ifo].append(event)
-				except KeyError:
-					eventlists[event.ifo] = [event]
-				orig_peak_times[event] = event.peak
+	@classmethod
+	def get_xml_root(cls, xml, name):
+		name = u"%s:%s" % (name, cls.ligo_lw_name_suffix)
+		xml = [elem for elem in xml.getElementsByTagName(ligolw.LIGO_LW.tagName) if elem.hasAttribute(u"Name") and elem.Name == name]
+		if len(xml) != 1:
+			raise ValueError("XML tree must contain exactly one %s element named %s" % (ligolw.LIGO_LW.tagName, name))
+		return xml[0]
 
-		# parse the --thresholds H1,L1=... command-line options from burca
-		delta_t = [float(threshold.split("=")[-1]) for threshold in ligolw_process.get_process_params(database.xmldoc, "ligolw_burca", "--thresholds")]
-		if not all(delta_t[0] == threshold for threshold in delta_t[1:]):
-			raise ValueError("\Delta t is not unique in ligolw_burca arguments")
-		delta_t = delta_t.pop()
+	@classmethod
+	def from_xml(cls, xml, name):
+		xml = cls.get_xml_root(xml, name)
+		self = cls([])
+		self.numerator = LnLRDensity.from_xml(xml, "numerator")
+		self.denominator = LnLRDensity.from_xml(xml, "denominator")
+		self.candidates = LnLRDensity.from_xml(xml, "candidates")
+		# FIXME:  stupid way to get instruments, store them in the
+		# XML
+		instruments = [key.replace("_snr2_chi2", "") for key in self.numerator.densities.keys() if "_snr2_chi2" in key]
+		self.triangulators = triangulators(dict.fromkeys(instruments, 8e-5))
+		return self
 
-		# construct the coinc generator.  note that H1+H2-only
-		# coincs are forbidden, which is affected here by removing
-		# that instrument combination from the object's internal
-		# .rates dictionary
-		coinc_generator = snglcoinc.CoincSynthesizer(eventlists, seglists, delta_t)
-		if frozenset(("H1", "H2")) in coinc_generator.rates:
-			del coinc_generator.rates[frozenset(("H1", "H2"))]
-
-		# build a dictionary of time-of-arrival generators
-		toa_generator = dict((instruments, coinc_generator.plausible_toas(instruments)) for instruments in coinc_generator.rates.keys())
-
-		# how many coincs?  the expected number is obtained by
-		# multiplying the total zero-lag time for which at least
-		# two instruments were on by the sum of the rates for all
-		# coincs to get the mean number of coincs per zero-lag
-		# observation time, and multiplying that by the number of
-		# experiments the background should simulate to get the
-		# mean number of background events to simulate.  the actual
-		# number simulated is a Poisson-distributed RV with that
-		# mean.
-		n_coincs, = scipy.stats.poisson.rvs(float(abs(segmentsUtils.vote(seglists.values(), 2))) * sum(coinc_generator.rates.values()) * experiments)
-
-		# generate synthetic background coincs
-		zero_lag_offset_vector = offsetvector((instrument, 0.0) for instrument in seglists)
-		for n, events in enumerate(coinc_generator.coincs(lsctables.SnglBurst.get_peak)):
-			# n = 1 on 2nd iteration, so placing this condition
-			# where it is in the loop causes the correct number
-			# of events to be added to the background
-			if n >= n_coincs:
-				break
-			# assign fake peak times
-			toas = toa_generator[frozenset(event.ifo for event in events)].next()
-			for event in events:
-				event.peak = toas[event.ifo]
-			# compute coincidence parameters
-			self.add_background(self.coinc_params(events, zero_lag_offset_vector, *param_func_args))
-
-		# restore original peak times
-		for event, peak_time in orig_peak_times.iteritems():
-			event.peak = peak_time
+	def to_xml(self, name):
+		xml = ligolw.LIGO_LW({u"Name": u"%s:%s" % (name, self.ligo_lw_name_suffix)})
+		xml.appendChild(self.numerator.to_xml("numerator"))
+		xml.appendChild(self.denominator.to_xml("denominator"))
+		xml.appendChild(self.candidates.to_xml("candidates"))
+		return xml
 
 
 #
@@ -317,10 +376,10 @@ def load_likelihood_data(filenames, verbose = False):
 	seglists = None
 	for n, filename in enumerate(filenames, 1):
 		if verbose:
-			print >>sys.stderr, "%d/%d:" % (n, len(filenames)),
-		xmldoc = ligolw_utils.load_filename(filename, verbose = verbose, contenthandler = StringCoincParamsDistributions.contenthandler)
+			print("%d/%d:" % (n, len(filenames)), end=' ', file=sys.stderr)
+		xmldoc = ligolw_utils.load_filename(filename, verbose = verbose, contenthandler = StringCoincParamsDistributions.LIGOLWContentHandler)
 		this_coinc_params = StringCoincParamsDistributions.from_xml(xmldoc, u"string_cusp_likelihood")
-		this_seglists = lsctables.SearchSummaryTable.get_table(xmldoc).get_out_segmentlistdict(set([this_coinc_params.process_id])).coalesce()
+		this_seglists = lsctables.SearchSummaryTable.get_table(xmldoc).get_out_segmentlistdict(lsctables.ProcessTable.get_table(xmldoc).get_ids_by_program(u"lalapps_string_meas_likelihood")).coalesce()
 		xmldoc.unlink()
 		if coinc_params is None:
 			coinc_params = this_coinc_params
@@ -333,10 +392,6 @@ def load_likelihood_data(filenames, verbose = False):
 	return coinc_params, seglists
 
 
-def write_likelihood_data(filename, coincparamsdistributions, seglists, verbose = False):
-	utils.write_filename(ligolw_burca_tailor.gen_likelihood_control(coincparamsdistributions, seglists, name = u"string_cusp_likelihood"), filename, verbose = verbose, gz = (filename or "stdout").endswith(".gz"))
-
-
 #
 # =============================================================================
 #
@@ -361,17 +416,17 @@ def time_slides_livetime(seglists, time_slides, min_instruments, verbose = False
 	seglists = seglists.copy()	# don't modify original
 	N = len(time_slides)
 	if verbose:
-		print >>sys.stderr, "computing the live time for %d time slides:" % N
+		print("computing the live time for %d time slides:" % N, file=sys.stderr)
 	for n, time_slide in enumerate(time_slides):
 		if verbose:
-			print >>sys.stderr, "\t%.1f%%\r" % (100.0 * n / N),
+			print("\t%.1f%%\r" % (100.0 * n / N), end=' ', file=sys.stderr)
 		seglists.offsets.update(time_slide)
 		if clip is None:
 			livetime += float(abs(segmentsUtils.vote(seglists.values(), min_instruments)))
 		else:
 			livetime += float(abs(segmentsUtils.vote((seglists & clip).values(), min_instruments)))
 	if verbose:
-		print >>sys.stderr, "\t100.0%"
+		print("\t100.0%", file=sys.stderr)
 	return livetime
 
 
@@ -388,10 +443,10 @@ def time_slides_livetime_for_instrument_combo(seglists, time_slides, instruments
 	offseglists = seglists.copy(keys = set(seglists) - set(instruments))
 	N = len(time_slides)
 	if verbose:
-		print >>sys.stderr, "computing the live time for %s in %d time slides:" % (", ".join(instruments), N)
+		print("computing the live time for %s in %d time slides:" % (", ".join(instruments), N), file=sys.stderr)
 	for n, time_slide in enumerate(time_slides):
 		if verbose:
-			print >>sys.stderr, "\t%.1f%%\r" % (100.0 * n / N),
+			print("\t%.1f%%\r" % (100.0 * n / N), end=' ', file=sys.stderr)
 		onseglists.offsets.update(time_slide)
 		offseglists.offsets.update(time_slide)
 		if clip is None:
@@ -399,7 +454,7 @@ def time_slides_livetime_for_instrument_combo(seglists, time_slides, instruments
 		else:
 			livetime += float(abs((onseglists & clip).intersection(onseglists.keys()) - offseglists.union(offseglists.keys())))
 	if verbose:
-		print >>sys.stderr, "\t100.0%"
+		print("\t100.0%", file=sys.stderr)
 	return livetime
 
 
@@ -412,23 +467,24 @@ def time_slides_livetime_for_instrument_combo(seglists, time_slides, instruments
 #
 
 
-def create_recovered_likelihood_table(connection, coinc_def_id):
+def create_recovered_ln_likelihood_ratio_table(connection, coinc_def_id):
 	"""
-	Create a temporary table named "recovered_likelihood" containing
-	two columns:  "simulation_id", the simulation_id of an injection,
-	and "likelihood", the highest likelihood ratio at which that
-	injection was recovered by a coincidence of type coinc_def_id.
+	Create a temporary table named "recovered_ln_likelihood_ratio"
+	containing two columns:  "simulation_id", the simulation_id of an
+	injection, and "ln_likelihood_ratio", the highest log likelihood
+	ratio at which that injection was recovered by a coincidence of
+	type coinc_def_id.
 	"""
 	cursor = connection.cursor()
 	cursor.execute("""
-CREATE TEMPORARY TABLE recovered_likelihood (simulation_id TEXT PRIMARY KEY, likelihood REAL)
+CREATE TEMPORARY TABLE recovered_ln_likelihood_ratio (simulation_id TEXT PRIMARY KEY, ln_likelihood_ratio REAL)
 	""")
 	cursor.execute("""
 INSERT OR REPLACE INTO
-	recovered_likelihood
+	recovered_ln_likelihood_ratio
 SELECT
 	sim_burst.simulation_id AS simulation_id,
-	MAX(coinc_event.likelihood) AS likelihood
+	MAX(coinc_event.likelihood) AS ln_likelihood_ratio
 FROM
 	sim_burst
 	JOIN coinc_event_map AS a ON (
